#!/usr/bin/env python3

# Internal packages
import unittest
import shutil
import sys
import tempfile
from pathlib import Path
from typing import List, Tuple, Optional

# Append the module path for md2anki
sys.path.append(str(Path(__file__).parent.parent.joinpath("src")))

# Local modules
from md2anki.cli import (
    convert_list_to_dict_merged,
    DEFAULT_CUSTOM_PROGRAMS,
    DEFAULT_CUSTOM_PROGRAM_ARGS,
    str_to_str,
    json_str_to_str_list,
)
from md2anki.info.general import MD2ANKI_NAME
from md2anki.evaluate_code import evaluate_code_in_subprocess


class TestEvaluateCode(unittest.TestCase):
    def setUp(self):
        self.code: List[Tuple[str, str]] = list()
        self.results: List[Tuple[List[str], List[Path]]] = list()
        self.expected: List[Tuple[Optional[List[str]], List[Path]]] = list()

        test_data: List[Tuple[str, str, Tuple[Optional[List[str]], List[Path]]]] = [
            ("", "py", ([""], [])),
            (
                'print("Hello world!")',
                "py",
                (["Hello world!\n"], []),
            ),
            (
                "a = 42\n" 'print(f"The answer is {a}")',
                "py",
                (["The answer is 42\n"], []),
            ),
            ("", "js", ([""], [])),
            (
                'console.log("Hello world!")',
                "js",
                (["Hello world!\n"], []),
            ),
            (
                "const a = 42\n" "console.log(`The answer is ${a}`)",
                "js",
                (["The answer is 42\n"], []),
            ),
            (
                'let message: string = "Hello world!"\nconsole.log(message)',
                "ts",
                (["Hello world!\n"], []),
            ),
            (
                "#include<stdio.h>\n"
                "int main() {\n"
                '	printf("Hello world!\\n");\n'
                "	return 0;\n"
                "}",
                "c",
                (["", "Hello world!\n"], []),
            ),
            (
                "#include <iostream>\n"
                "int main() {\n"
                '	std::cout << "Hello world!" << std::endl;\n'
                "	return 0;\n"
                "}",
                "cpp",
                (["", "Hello world!\n"], []),
            ),
            (
                "import numpy as np\n"
                "import matplotlib.pyplot as plt\n"
                "\n"
                "def m_t_exp_wachstum(c, k, t):\n"
                "    return c * np.exp(k * t)\n"
                "\n"
                "def m_t_exp_wachstum_abl(c, k, t):\n"
                "    return k * m_t_exp_wachstum(c, k, t)\n"
                "\n"
                "x = np.linspace(0, 200)\n"
                "y_m_t = m_t_exp_wachstum(c=30, k=-0.04, t=x)\n"
                "y_m_t_abl = m_t_exp_wachstum_abl(c=30, k=-0.04, t=x)\n"
                "\n"
                "plt.plot(x, y_m_t, '-b', label='$M_{vereinfacht}(t)$')\n"
                "plt.xlabel('$t$ (in min)')\n"
                "plt.legend(loc='upper right')\n"
                "plt.savefig('graph_3.svg')\n",
                "py",
                ([""], [Path("graph_3.svg")]),
            ),
        ]

        if sys.platform != "win32":
            test_data.append(
                (
                    "\\documentclass[tikz,border=10pt]{standalone}\n"
                    "\\usetikzlibrary{positioning}\n"
                    "\\tikzset{\n"
                    "    main\n"
                    "    node/.style={circle,fill=none,draw,minimum size=1cm,inner sep=0pt}\n"
                    "}\n"
                    "\\begin{document}\n"
                    "\\begin{tikzpicture}\n"
                    "    \\node[main node] [                               ] (0)  {$A$};\n"
                    "    \\node[main node] [right       = 2cm          of 0] (1)  {$B$};\n"
                    "\n"
                    "     \\path[draw,thick]\n"
                    "     (0) edge node {} (1)\n"
                    "     ;\n"
                    "\\end{tikzpicture}\n"
                    "\\end{document}\n",
                    "latex",
                    (None, [Path("code.svg")]),
                ),
            )

        for test_input, test_program, test_expected in test_data:
            self.code.append((test_program, test_input))
            dir_dynamic_files = Path(
                tempfile.mkdtemp(prefix=f"{MD2ANKI_NAME}_tmp_file_test_subprocess_")
            )
            self.results.append(
                evaluate_code_in_subprocess(
                    test_program,
                    test_input,
                    custom_program=convert_list_to_dict_merged(
                        DEFAULT_CUSTOM_PROGRAMS, str_to_str
                    ),
                    custom_program_args=convert_list_to_dict_merged(
                        DEFAULT_CUSTOM_PROGRAM_ARGS, json_str_to_str_list
                    ),
                    dir_dynamic_files=dir_dynamic_files,
                )
            )
            shutil.rmtree(dir_dynamic_files)
            self.expected.append(
                (
                    test_expected[0],
                    [dir_dynamic_files.joinpath(x) for x in test_expected[1]],
                )
            )

    def test_evaluated_code_output_matches(self):
        for (language, code), result, expected in zip(
            self.code, self.results, self.expected
        ):
            with self.subTest(language=language, code=code):
                if expected[0] is not None:
                    self.assertEqual(
                        result[0],
                        expected[0],
                        f"Check if output stdout {result[0]=}=={expected[0]=}",
                    )
                self.assertEqual(
                    result[1],
                    expected[1],
                    f"Check if output image files {result[1]=}=={expected[1]=}",
                )



#!/usr/bin/env python3

# Internal packages
from pathlib import Path
from typing import Final

# Relative file paths to res files
RELATIVE_RES_DIR: Final = Path("res")
RELATIVE_RES_CSS_FILE_PATH: Final = RELATIVE_RES_DIR.joinpath("stylesheet.css")
RELATIVE_RES_CSS_FILE_PATH_TYPE_ANSWER: Final = RELATIVE_RES_DIR.joinpath(
    "stylesheet_type_answer.css"
)
RELATIVE_RES_CSS_FILE_PATH_TYPE_CLOZE: Final = RELATIVE_RES_DIR.joinpath(
    "stylesheet_type_cloze.css"
)



#!/usr/bin/env python3

# Internal packages
import logging
import os
import shutil
import subprocess
from pathlib import Path
from typing import List, Optional, Final, Dict

log = logging.getLogger(__name__)


class ProgramNotFoundException(Exception):
    """Raised when a program cannot be found"""

    pass


class ProgramExitedWithWarningsException(Exception):
    """Raised when a program exists with warnings"""

    pass


def run_subprocess(
    command: str,
    args: Optional[List[str]] = None,
    cwd: Optional[Path] = None,
    additional_env: Optional[Dict[str, str]] = None,
):
    """
    Run an external process sequentially.

    @param command: Command/exe that should be run.
    @param args: Optional arguments of command.
    @param cwd: Optional directory from which the command should be run.
    @param additional_env: Optional additional environment variables with which the command should be run.
    @return:
    """
    command_path: Optional[str] = None
    if cwd is not None:
        command_path = shutil.which(str(cwd.joinpath(command)))
    if command_path is None and Path(command).is_file() and os.access(command, os.X_OK):
        command_path = command
        log.debug(f"Found command path os.access=OK {command_path=}")
    if command_path is None:
        command_path = shutil.which(command)
        log.debug(f"Found command path shutil.which {command_path=}")
    if command_path is None:
        raise ProgramNotFoundException(f"{command=} could not be found")
    command_args: Final[List[str]] = list() if args is None else args
    command_env = os.environ.copy()
    if additional_env is not None:
        for key, value in additional_env.items():
            command_env[key] = value
    log.debug(
        f"Run subprocess {command=}/{command_path=} {command_args=} ({cwd=},{additional_env=})"
    )
    p = subprocess.run(
        [command_path] + command_args,
        capture_output=True,
        text=True,
        cwd=cwd,
        env=command_env,
    )
    log.debug(f"Subprocess exited {p.stdout=} {p.returncode=}")
    if p.returncode != 0:
        raise ProgramExitedWithWarningsException(
            f"Subprocess exited with warnings ({command=},{args=},{cwd=},"
            f"{p.returncode=},{p.stdout=},{p.stderr=})"
        )
    return p.stdout



#!/usr/bin/env python3

# Open the entry point when program is executed as a module using 'python -m md2anki [...]'

# Local modules
from md2anki.entry_point import entry_point


if __name__ == "__main__":
    entry_point()



#!/usr/bin/env python3

# Internal packages
from enum import Enum
from pathlib import Path
from typing import Final

# Local modules
from md2anki.anki_model import AnkiModel
from md2anki.info.files import (
    RELATIVE_RES_CSS_FILE_PATH,
    RELATIVE_RES_CSS_FILE_PATH_TYPE_ANSWER,
    RELATIVE_RES_CSS_FILE_PATH_TYPE_CLOZE,
)
from md2anki.info.general import MD2ANKI_NAME

CURRENT_DIR: Final = Path(__file__).parent
CSS_FILE_PATH: Final = CURRENT_DIR.joinpath(RELATIVE_RES_CSS_FILE_PATH)
CSS_FILE_PATH_TYPE_ANSWER: Final = CURRENT_DIR.joinpath(
    RELATIVE_RES_CSS_FILE_PATH_TYPE_ANSWER
)
CSS_FILE_PATH_TYPE_CLOZE: Final = CURRENT_DIR.joinpath(
    RELATIVE_RES_CSS_FILE_PATH_TYPE_CLOZE
)


class AnkiCardModelId(Enum):
    DEFAULT = f"{MD2ANKI_NAME}_type_default"
    TYPE_ANSWER = f"{MD2ANKI_NAME}_type_answer"
    TYPE_CLOZE = f"{MD2ANKI_NAME}_type_cloze"
    TYPE_CLOZE_EXTRA = f"{MD2ANKI_NAME}_type_cloze_extra"

    def __str__(self):
        return self.value


def create_type_answer_anki_deck_model() -> AnkiModel:
    with open(CSS_FILE_PATH, "r") as file:
        css_data = file.read()
    with open(CSS_FILE_PATH_TYPE_ANSWER, "r") as file:
        css_data += file.read()
    return AnkiModel(
        guid=396500203,
        name=f"{MD2ANKI_NAME} {AnkiCardModelId.TYPE_ANSWER.value} (v3)",
        description=f"{MD2ANKI_NAME} (type answer)",
        css=css_data,
        template_card_question='<div class="card card_question">{{Question}}</div>\n\n{{type:Answer}}',
        template_card_question_surround=("", ""),
        template_card_answer_front_side='<div class="card card_question">{{Question}}</div>\n\n<hr id="answer">\n\n',
        template_card_answer="{{type:Answer}}",
    )


def create_type_cloze_anki_deck_model(extra=False) -> AnkiModel:
    with open(CSS_FILE_PATH, "r") as file:
        css_data = file.read()
    with open(CSS_FILE_PATH_TYPE_CLOZE, "r") as file:
        css_data += file.read()
    return AnkiModel(
        guid=396500402 if extra else 396500302,
        name=f"{MD2ANKI_NAME} {AnkiCardModelId.TYPE_CLOZE_EXTRA.value if extra else AnkiCardModelId.TYPE_CLOZE.value} (v2)",
        description=f"{MD2ANKI_NAME} (type cloze{' extra' if extra else ''})",
        fields=["Text", "Extra"] if extra else ["Text"],
        css=css_data,
        cloze=True,
        template_card_question="{{cloze:Text}}",
        template_card_answer='{{cloze:Text}}<hr id="answer">{{Extra}}'
        if extra
        else "{{cloze:Text}}",
        template_card_answer_front_side="",
    )


def create_default_anki_deck_model() -> AnkiModel:
    with open(CSS_FILE_PATH, "r") as file:
        css_data = file.read()
    return AnkiModel(
        guid=396500103,
        name=f"{MD2ANKI_NAME} {AnkiCardModelId.DEFAULT.value} (v3)",
        description=f"{MD2ANKI_NAME} (default)",
        css=css_data,
    )



#!/usr/bin/env python3

# Internal packages
from dataclasses import dataclass, field
from typing import List, Optional, Tuple

# Installed packages
import genanki

# Local modules
from md2anki.create_id import create_unique_id_int


@dataclass
class AnkiModel:
    """
    Contains all information of an anki model (for anki notes of an anki deck)
    """

    name: str = ""
    """Model name"""
    description: str = ""
    """Model description"""
    fields: List[str] = field(default_factory=lambda: ["Question", "Answer"])
    """The content fields"""
    cloze: bool = False
    """Use the cloze model type"""
    css: str = ""
    """Style information (CSS code) for each anki note"""
    js: Optional[str] = None
    """Script information (JS code) for each anki note"""
    guid: int = create_unique_id_int()
    """Unique id of anki model"""
    template_card_question: str = "{{Question}}"
    """Card question template"""
    template_card_question_surround: Tuple[str, str] = (
        '<div class="card card_question">',
        "</div>",
    )
    """Card question template content before and after"""
    template_card_answer_front_side: str = '{{FrontSide}}<hr id="answer">'
    """Card answer front side"""
    template_card_answer: str = "{{Answer}}"
    """Card answer text"""
    template_card_answer_surround: Tuple[str, str] = (
        '<div class="card card_answer">',
        "</div>",
    )
    """Card answer template content before and after"""

    def genanki_create_model(self) -> genanki.Model:
        js_string = (
            ""
            if self.js is None
            else f'<script type="text/javascript>{self.js}</script>'
        )
        return genanki.Model(
            self.guid,
            self.description,
            fields=[dict([("name", x)]) for x in self.fields],
            css=self.css,
            templates=[
                {
                    "name": self.name,
                    "qfmt": f"{js_string}\n{self.template_card_question_surround[0]}\n{self.template_card_question}\n"
                    f"{self.template_card_question_surround[1]}",
                    "afmt": f"{self.template_card_answer_front_side}{self.template_card_answer_surround[0]}\n"
                    f"{self.template_card_answer}\n{self.template_card_answer_surround[1]}",
                }
            ],
            model_type=genanki.Model.CLOZE if self.cloze else genanki.Model.FRONT_BACK,
        )



#!/usr/bin/env python3

# Internal packages
import logging
import shutil
import tempfile
from os import fdopen
from pathlib import Path
from typing import List, Dict, Final, Tuple, Optional

# Local modules
from md2anki.info.general import MD2ANKI_NAME
from md2anki.preprocessor import md_preprocessor_md2anki
from md2anki.subprocess import run_subprocess


PANDOC_ARGS_PDF_KEY_NAME: Final = "pandoc_pdf"
PANDOC_ARGS_PDF: Final[Dict[str, List[Tuple[str, List[str]]]]] = {
    PANDOC_ARGS_PDF_KEY_NAME: [
        (
            "pandoc",
            [
                "--from",
                "markdown",
                "--to",
                "pdf",
                "--table-of-contents",
                "-V",
                "geometry:a4paper",
                "-V",
                "geometry:margin=2cm",
                "--pdf-engine=xelatex",
                "--pdf-engine-opt=-shell-escape",
            ],
        )
    ]
}
"""Pandoc args for pdf export."""

log = logging.getLogger(__name__)


def create_pdf_from_md_content(
    md_content: str,
    output_file_path: Path,
    local_assets: List[Path],
    custom_program: Dict[str, List[str]],
    custom_program_args: Dict[str, List[List[str]]],
    dir_dynamic_files: Path,
    external_file_dirs: List[Path],
    evaluate_code: bool = False,
    evaluate_code_cache_dir: Optional[Path] = None,
    keep_temp_files: bool = False,
):
    md_content = md_preprocessor_md2anki(
        md_content,
        dir_dynamic_files=dir_dynamic_files,
        custom_program=custom_program,
        custom_program_args=custom_program_args,
        evaluate_code=evaluate_code,
        evaluate_code_cache_dir=evaluate_code_cache_dir,
        external_file_dirs=external_file_dirs,
        keep_temp_files=keep_temp_files,
    )

    fd, md_file_path_temp_str = tempfile.mkstemp(
        prefix=f"{MD2ANKI_NAME}_tmp_file_pandoc_md_", suffix=".md"
    )
    md_file_path_temp: Final = Path(md_file_path_temp_str)
    asset_dir_path_temp: Final = Path(
        tempfile.mkdtemp(prefix=f"{MD2ANKI_NAME}_tmp_file_pandoc_md_assets_")
    )
    try:
        log.debug(f"Copy {local_assets=} to {asset_dir_path_temp=}")
        for local_asset in local_assets:
            log.debug(f"> Copy {local_asset=} to {asset_dir_path_temp=}")
            shutil.copy2(local_asset, asset_dir_path_temp)
        log.debug(f"Copy {dir_dynamic_files=} to {asset_dir_path_temp=}")
        for dir_dynamic_file in dir_dynamic_files.rglob("*"):
            log.debug(f"> Copy {dir_dynamic_file=} to {asset_dir_path_temp=}")
            shutil.copy2(dir_dynamic_file, asset_dir_path_temp)
        with fdopen(fd, "w") as tmp:
            tmp.write(md_content)
        pandoc = custom_program[PANDOC_ARGS_PDF_KEY_NAME][0]
        pandoc_args = custom_program_args[PANDOC_ARGS_PDF_KEY_NAME][0]
        cli_args: List[str] = [
            *pandoc_args,
            "-f",
            "markdown",
            str(md_file_path_temp),
            "-o",
            str(output_file_path.resolve()),
        ]
        subprocess_stdout = run_subprocess(pandoc, cli_args, cwd=asset_dir_path_temp)
        log.debug(f"> Stdout:         {subprocess_stdout=}")
    finally:
        if not keep_temp_files:
            md_file_path_temp.unlink()
            shutil.rmtree(asset_dir_path_temp)



#!/usr/bin/env python3

# Internal packages
from typing import Final, Tuple

# anki information
ANKI_SUBDECK_SEPARATOR: Final = "::"
"""This seperator in the name of a deck marks it as a subdeck: `GrandparentDeck::ParentDeck::ChildDeck`"""
ANKI_CLOZE_SYNTAX_SYMBOLS: Final[Tuple[str, str, str, str]] = ("{{c", "::", "::", "}}")
"""This is a list for all cloze syntax symbols (e.g. `{{c1::text::hint}}`)"""
ANKI_SUPPORTED_FILE_FORMATS: Final = [".jpg", ".png", ".svg"]
"""A list of files that are natively supported as assets"""



#!/usr/bin/env python3

# Internal packages
import logging
import shutil
import tempfile
from pathlib import Path
from typing import Dict, Final, List, Tuple

# Local modules
from md2anki.anki_deck import (
    AnkiDeck,
    AnkiNote,
    genanki_package_anki_decks_to_file,
    md_merge_anki_decks_to_md_file,
    backup_anki_decks_to_dir,
)
from md2anki.cli import Md2AnkiArgs, AnkiCardModelId
from md2anki.info.general import MD2ANKI_NAME
from md2anki.md_parser import parse_md_content_to_anki_deck_list
from md2anki.md_to_pdf import create_pdf_from_md_content
from md2anki.note_models import (
    create_default_anki_deck_model,
    create_type_answer_anki_deck_model,
    create_type_cloze_anki_deck_model,
)

log = logging.getLogger(__name__)


def main(args: Md2AnkiArgs) -> int:
    log.debug(f"{args=}")

    # Check if there were any critical errors when parsing the CLI args
    if args.error is not None:
        log.error(args.error)
        return 1

    # Other routines
    if (
        args.evaluate_code_delete_cache
        and args.evaluate_code_cache_dir_path is not None
    ):
        log.debug(
            f"Delete evaluate code cache dir {args.evaluate_code_cache_dir_path!r}"
        )
        if args.evaluate_code_cache_dir_path.is_dir():
            shutil.rmtree(args.evaluate_code_cache_dir_path)
    if args.evaluate_code_cache_dir_path is not None and args.evaluate_code:
        args.evaluate_code_cache_dir_path.mkdir(parents=True, exist_ok=True)

    # Stop parsing if no outputs are specified
    if (
        args.anki_output_file_path is None
        and args.md_output_file_paths is None
        and args.md_output_dir_path is None
        and args.pdf_output_file_path is None
        and args.backup_output_dir_path is None
    ):
        log.warning("Stop parsing because no outputs are specified!")
        return 0

    # Parse all Markdown input files to Anki decks
    anki_decks: Final[List[List[AnkiDeck]]] = []
    for md_input_file_path in args.md_input_file_paths:
        log.debug(f"Parse {md_input_file_path!r} to anki deck list...")
        with open(md_input_file_path, "r", encoding="utf-8") as md_file:
            anki_deck_list = parse_md_content_to_anki_deck_list(
                md_file,
                [md_input_file_path.parent],
                initial_heading_depth=args.md_heading_depth,
            )
            for anki_deck in anki_deck_list:
                anki_deck.additional_file_dirs.extend(args.additional_file_dirs)
            anki_decks.append(anki_deck_list)

    anki_decks_flat: Final[List[AnkiDeck]] = [
        item for sublist in anki_decks for item in sublist
    ]

    guids: Dict[str, Tuple[AnkiDeck, AnkiNote]] = dict()
    guids_deck: Dict[int, AnkiDeck] = dict()
    for anki_deck in anki_decks_flat:
        if (
            anki_deck.guid in guids_deck
            and guids_deck[anki_deck.guid].name != anki_deck.name
        ):
            log.warning(
                f"Found a duplicated deck guid with different name ({anki_deck.guid}): "
                f"{guids_deck[anki_deck.guid].name!r} <-> {anki_deck.name!r}"
            )
        guids_deck[anki_deck.guid] = anki_deck
        for anki_deck_note in anki_deck.notes:
            if anki_deck_note.guid in guids:
                log.warning(
                    f"Found a duplicated note guid ({anki_deck_note.guid}): "
                    f"{guids[anki_deck_note.guid][0].name!r}>{guids[anki_deck_note.guid][1].question!r} <-> "
                    f"{anki_deck.name!r}>{anki_deck_note.question!r}"
                )
            guids[anki_deck_note.guid] = (anki_deck, anki_deck_note)

    if args.anki_output_file_path is not None:
        log.debug(f"Create anki deck file {args.anki_output_file_path!r}...")
        tmp_dir_dynamic_files_anki = Path(
            tempfile.mkdtemp(
                prefix=f"{MD2ANKI_NAME}_tmp_dir_dynamic_files_anki_output_"
            )
        )
        try:
            genanki_package_anki_decks_to_file(
                [
                    anki_deck.genanki_create_anki_deck(
                        default_anki_card_model=f"{args.anki_card_model}",
                        dir_dynamic_files=tmp_dir_dynamic_files_anki,
                        custom_program=args.custom_program,
                        custom_program_args=args.custom_program_args,
                        evaluate_code=args.evaluate_code,
                        evaluate_code_cache_dir=None
                        if args.evaluate_code_ignore_cache
                        else args.evaluate_code_cache_dir_path,
                        external_file_dirs=args.additional_file_dirs,
                        keep_temp_files=args.keep_temp_files,
                    )
                    for anki_deck in anki_decks_flat
                ],
                args.anki_output_file_path,
            )
        except Exception as err:
            log.error(err)
            raise err
        finally:
            if not args.keep_temp_files:
                shutil.rmtree(tmp_dir_dynamic_files_anki)

    if args.md_output_file_paths is not None:
        log.debug(f"Create markdown file {args.md_output_file_paths!r}...")
        # Don't update local file paths when merging or updating files
        if len(args.md_output_file_paths) == 1:
            md_merge_anki_decks_to_md_file(
                anki_decks_flat,
                args.md_output_file_paths[0],
                initial_heading_depth=args.md_heading_depth,
            )
        elif len(args.md_output_file_paths) > 1:
            if len(args.md_output_file_paths) != len(anki_decks):
                raise RuntimeError(
                    f"Output file path ({len(args.md_output_file_paths)}) and anki deck list ({len(anki_decks)}) had different sizes"
                )
            for output_file_path, anki_deck_list in zip(
                args.md_output_file_paths, anki_decks
            ):
                md_merge_anki_decks_to_md_file(
                    anki_deck_list,
                    output_file_path,
                    initial_heading_depth=args.md_heading_depth,
                )

    if args.md_output_dir_path is not None:
        log.debug(f"Create markdown files in {args.md_output_dir_path!r}...")
        if not args.md_output_dir_path.exists():
            args.md_output_dir_path.mkdir()
        for anki_deck_list, md_input_file_path in zip(
            anki_decks, args.md_input_file_paths
        ):
            md_merge_anki_decks_to_md_file(
                anki_deck_list,
                args.md_output_dir_path.joinpath(f"{md_input_file_path.stem}.apkg"),
                initial_heading_depth=args.md_heading_depth,
            )

    if args.backup_output_dir_path is not None:
        log.debug(f"Create backup in {args.backup_output_dir_path!r}...")
        backup_anki_decks_to_dir(
            anki_decks,
            args.backup_output_dir_path,
            initial_heading_depth=args.md_heading_depth,
        )

    if args.pdf_output_file_path is not None:
        log.debug(f"Create pdf in {args.backup_output_dir_path!r}...")
        tmp_dir_dynamic_files_pdf: Final = Path(
            tempfile.mkdtemp(prefix=f"{MD2ANKI_NAME}_tmp_dir_dynamic_files_pdf_output_")
        )
        tmp_file_path_md_merge: Final = Path(
            tempfile.mktemp(prefix=f"{MD2ANKI_NAME}_tmp_file_md_merge_pdf_output_")
        )
        try:
            md_merge_anki_decks_to_md_file(
                anki_decks_flat,
                tmp_file_path_md_merge,
                local_asset_dir_path=Path("."),
                remove_ids=True,
            )
            local_assets: List[Path] = []
            for anki_deck in anki_decks_flat:
                local_assets.extend(anki_deck.get_local_files_from_notes())
            with open(tmp_file_path_md_merge, "r") as reader:
                create_pdf_from_md_content(
                    reader.read(),
                    args.pdf_output_file_path,
                    local_assets,
                    custom_program=args.custom_program,
                    custom_program_args=args.custom_program_args,
                    dir_dynamic_files=tmp_dir_dynamic_files_pdf,
                    evaluate_code=args.evaluate_code,
                    evaluate_code_cache_dir=None
                    if args.evaluate_code_ignore_cache
                    else args.evaluate_code_cache_dir_path,
                    external_file_dirs=args.additional_file_dirs,
                    keep_temp_files=args.keep_temp_files,
                )
        except Exception as err:
            log.error(err)
            raise err
        finally:
            if not args.keep_temp_files:
                tmp_file_path_md_merge.unlink()
                shutil.rmtree(tmp_dir_dynamic_files_pdf)

    return 0
